/*________________________  gp.photo.dat.c   _______________________*/

/*       1         2         3         4         5         6        */
/*345678901234567890123456789012345678901234567890123456789012345678*/
/*********************************|**********************************/

/*********************************|**********************************/
/*                                                                  */
/* Copyright (c) 2011-2013 SEIKO EPSON CORP.   All Rights Reserved  */
/*                                                                  */
/*                    Common Public License - v 1.0                 */
/*   THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF THIS   */
/*   COMMON PUBLIC LICENSE ("AGREEMENT"). ANY USE x REPRODUCTION OR */
/*   DISTRIBUTION OF THE PROGRAM CONSTITUTES RECIPIENT'S ACCEPTANCE */
/*   OF THIS AGREEMENT.                                             */
/*                                                                  */
/*   The Common Public License in its entirety is located in        */
/*                                                                  */
/*                    gpCommonPublicLicense.txt                     */
/*                                                                  */
/*   and applies to the contents of this file.                      */
/*                                                                  */
/*******%*******************%*****|**********************************/
/*                                                                  */
/* Code States:                                                     */
/*      Initial                                                     */
/*      Developing                                                  */
/*      Working                                                     */
/*      Tested                                                      */
/*      Refactored                                                  */
/*      Final Candidate                                             */
/*                          Final Code                              */
/*                                                                  */
/*******%*******************%*****|**********************************/

/*---------------------------  Includes   --------------------------*/
/*................................|.................................*/
#include "gw.win.h"
#include "gp.app.print.h"
#include "gp.print.preview.file.h"
#include "gp.media.dat.h"

#include "pwg.pjt.xxx.N.media.h"
#include "pwg.pjt.pdp.1.orientation.requested.h"
#include "pwg.pjt.pdp.1.sides.h"

/*-------------------------- Data Types ----------------------------*/
/*................................|.................................*/

/*------------------------  Generic Macros   -----------------------*/
/*................................|.................................*/

/*-------------------------  Definitions  --------------------------*/
/*................................|.................................*/

/*** 200 pix / [ min < w-pix, h-pix > ]  ==> w-pix = w-inch * ppi   */
/*** for MS, the effective ppi is 109 for width and 103 for height  */
/*** in theory the ratio is just the pageSize ratio.                */
/*** for 8.5 x 11.0 => 927 x 1133; with min == 927 =>               */
/*** scale = 200/927 => letterImageWidth  = 200;                    */
/***                    letterImageHeight = 200*1133/927 = 245      */
//#define LETTER_IMAGE_WIDTH    927 based on MS display in Notepad
#define LETTER_IMAGE_WIDTH    (85*96)/10 + 10
#define LETTER_IMAGE_HEIGHT   1133

/*------------------  Data Structure Declarations   ----------------*/
/*................................|.................................*/

/*-----------------  Application Global Variables   ----------------*/
/*................................|.................................*/

/*--------------------  Local Global Variables  --------------------*/
/*................................|.................................*/


/*--------------------  Function Declarations   --------------------*/
/*................................|.................................*/
// static gpERR_CODE _WritePpfHeader(FILE*, gpPrintPreviewData*);

/*********************************|**********************************/
/* Function name: AppPrint                                          */
/*********************************|**********************************/
gpERR_CODE AppPrint(

    gpBIT8*   pages,
    gpBIT32   totalPages,
    gpBIT32   pageSize

) {
/*** Declare Variable Local to Routine                              */
/*** -------------------------------------------------------------- */
gpERR_CODE          status;
gpPrintPreviewData  ppd;
gpBIT8*             aDataPage;
gpBIT32             idx, jdx;
gpCHAR              fileName[128];
gpBIT8*             ptr;
FILE*               ppfFile;
gpBIT32             scaledHeight;
gpBIT8              fill[128];

/*** Initialize Local Variables                                     */
/*** -------------------------------------------------------------- */
    status = gpERROR_NONE;
    memset( &ppd,'\0', sizeof(gpPrintPreviewData));
    memset(&fill, 0, 128);
    ptr = gpNULL;
    scaledHeight = 1;

/*** Simple Mechanism for error control                             */
/*** -------------------------------------------------------------- */
    while (gpTRUE) {

/*** File in the Print Preview Data                                 */
/***      gpBIT32     fileIndicator;                                */
/***      gpBIT32     version;                                      */
/***      gpPrintContentParametricData	pCPD;                       */
/***          gpBIT32     fileIndicator;                            */
/***          gpBIT32     version;                                  */
/***          gpCHAR      printContentUri[1024];                    */
/***          gpBIT32     numPages;                                 */
/***          gpBIT32     mediaWidth;                               */
/***          gpBIT32     mediaHeight;                              */
/***          gpBIT32     mediaColor;                               */
/***          gpBIT32     orientation;                              */
/***          gpBIT32     sides;                                    */
/***          gpLocation  location;                                 */
/***              gpINT32		latitude;                               */
/***              gpINT32		longitude;                              */
/***              gpINT32		altitude;                               */
/***              gpCHAR		namedLocation[1024];                    */
/***              gpBIT8		localNetworkDomain[4];                  */
/***          gpTime      time;                                     */
/***              gpBIT16		year;                                   */
/***              gpBIT16		month;                                  */
/***              gpBIT16		day;                                    */
/***              gpBIT16		hour;                                   */
/***              gpBIT16		minutes;                                */
/***              gpBIT16		seconds;                                */
/***      gpBIT32     numPages;                                     */
/***      gpBIT32     ppImageWidth;                                 */
/***      gpBIT32     ppImageHeight;                                */
/***      gpBIT32     mediaWidth;                                   */
/***      gpBIT32     mediaHeight;                                  */
/***      gpBIT32     mediaColor;                                   */
/***      gpBIT32     orientation;                                  */
/***      gpPpImage** pagePreviews;                                 */
/***          gpBIT8		reserved[128]; * N                          */
/***          gpBIT8*		pageData;      * N                          */

/*** -------------------------------------------------------------- */
                      /*  01234567                                  */
    ppd.fileIndicator = 0x5F505046;     /*  File Indicator          */
                      /*   "_PPF"                                   */

    ppd.version       = VERSION_AS_NUM(  1,   0);
                      /*                 ^    ^--- Minor Version    */
                      /*                 +--- Major Version         */

    /* Print Content Parametric Data                                */
    /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .  */
        ppd.pCPD.fileIndicator =
                      /*  01234567                                  */
                        0x5F504346;     /*  File Indicator          */
                      /* "_PCF"                                     */

        ppd.pCPD.version = VERSION_AS_NUM(  1,   0);
                      /*                 ^    ^--- Minor Version    */
                      /*                 +--- Major Version         */

        strcpy(ppd.pCPD.printContentUri,"ThisDir/ThisFile\0");

        ppd.pCPD.numPages    = totalPages;
        //ppd.pCPD.mediaWidth  = pwgMediaSizeName_KWVAlues[PWG_MEDIA_SIZE_NAME_NA_LETTER_8o5X11IN].mm.width /10;
        //ppd.pCPD.mediaHeight = pwgMediaSizeName_KWVAlues[PWG_MEDIA_SIZE_NAME_NA_LETTER_8o5X11IN].mm.height/10;
        //ppd.pCPD.mediaColor  = gpStdMediaColor[gpMEDIA_COLOR_WHITE].rgba;

        ppd.pCPD.orientation = PWG_ORIENTATION_REQUESTED_PORTRAIT;
        ppd.pCPD.sides       = PWG_SIDES_ONE_SIDED;

            /* My Current Location:                               */
            /* Latitude:    37.374373 ( 37° 22' 27.74'' N)        */
            /* Longitude: -121.912694 (121° 54' 45.70'' W)        */
            /* Altitude:    43.00 ft = 13.1064 meters             */
        ppd.pCPD.location.latitude  =   37374373;
        ppd.pCPD.location.longitude = -121912694;
        ppd.pCPD.location.altitude  =     131064;
        strcpy(ppd.pCPD.location.namedLocation,"This Place\0");
        ppd.pCPD.location.localNetworkDomain = 0x72717969;

            /* Time is 2012.07.18 - 22:25:17 (GMT)                */
        ppd.pCPD.time.year    = 2012;
        ppd.pCPD.time.month   =    7;
        ppd.pCPD.time.day     =   18;
        ppd.pCPD.time.hour    =   22;
        ppd.pCPD.time.minutes =   25;
        ppd.pCPD.time.seconds =   17;

    ppd.numPages          = totalPages;
    ppd.ppImageWidth      = LETTER_IMAGE_WIDTH;
    ppd.ppImageHeight     = LETTER_IMAGE_HEIGHT;
    //ppd.mediaWidth        = gpStdMediaSize[NA_LETTER_8o5X11IN].mm.width /10;
    //ppd.mediaHeight       = gpStdMediaSize[NA_LETTER_8o5X11IN].mm.height/10;
    //ppd.mediaColor        = gpStdMediaColor[gpMEDIA_COLOR_WHITE].rgba;
    ppd.pCPD.orientation  = PWG_ORIENTATION_REQUESTED_PORTRAIT;

    ppfFile = (FILE*)fopen("test.ppf", "wb+");
    if (ppfFile == gpNULL) {status = gpERROR_OPR_FAIL; break;}

/*** Replaced the R/W of individual element with a struct write     */
//    status = _WritePpfHeader(ppfFile, &ppd);
//    if (status != gpERROR_NONE) { fclose(ppfFile); break; }
/*** .............................................................. */
    fwrite((const void*)&ppd,1,sizeof(gpPrintPreviewData),ppfFile);

    for (idx = 0; idx < ppd.numPages; idx++) {
        /* Write "reserved" 128 bytes before image data             */
        /* For testing, modified content from zeros to the image    */
        /* page number.                                             */
        memset(&fill, idx+1, 128);
        fwrite(&fill,1,128,ppfFile);

        aDataPage = (gpBIT8*)&pages[idx*pageSize];

        sprintf(fileName, "ppf_page_%d.png", idx+1);
        PrtVal("Converting Page %d to Preview Image---\n", idx+1);

        gwTextToImage(aDataPage,
                      LETTER_IMAGE_WIDTH,
                      LETTER_IMAGE_HEIGHT,
                      fileName);

/*** ...............................................................*/
/*** GWP Added for Debug only - do not depend of code               */
#if 0
        sprintf(fileName, "ppf_base_page_%d.png", idx+1);
        gwBaseImg->save(fileName);
#endif
/*** ...............................................................*/

        scaledHeight = (200*LETTER_IMAGE_HEIGHT)/LETTER_IMAGE_WIDTH;
        for (jdx = 0; jdx < scaledHeight; jdx++) {
          gwGetImageLine(ptr, jdx);
//PrtVal("Ptr: %x\n",ptr);
          fwrite((const void*)ptr,1,200*4,ppfFile);
        }
    }

  Prt("---Print Button---");

/*** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/*** Did an Error occur???                                          */
/*** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
        if (ppfFile) fclose(ppfFile);

        break;
    }

/*** Return to Caller                                               */
/*** -------------------------------------------------------------- */
    return(status);

} /* AppPrint                                                       */

#if 0
/*********************************|**********************************/
/* Function name: _WritePpfHeader                                   */
/*********************************|**********************************/
static gpERR_CODE _WritePpfHeader(

      FILE*               file,
      gpPrintPreviewData* ppd

) {
/*** Declare Variable Local to Routine                              */
/*** -------------------------------------------------------------- */
gpERR_CODE          status;

#define WRT(d,s)    fwrite((const void*)&d,1,s,file)

/*** Initialize Local Variables                                     */
/*** -------------------------------------------------------------- */
    status = gpERROR_NONE;


/*** Simple Mechanism for error control                             */
/*** -------------------------------------------------------------- */
    while (gpTRUE) {

/*** Input Data check                                               */
/*** -------------------------------------------------------------- */
    if (file == gpNULL) { status = gpERROR_INVALID_ARG; break; }
    if (ppd  == gpNULL) { status = gpERROR_INVALID_ARG; break; }

/*** Start Writing PPF Header info                                  */
/*** -------------------------------------------------------------- */
    rewind(file);

    WRT(ppd->fileIndicator  , 4);
    WRT(ppd->version        , 4);

    /* Print Content Parametric Data                                */
    /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .  */
        WRT(ppd->pCPD.fileIndicator  ,    4);
        WRT(ppd->pCPD.version        ,    4);

        WRT(ppd->pCPD.printContentUri, 1024);

        WRT(ppd->pCPD.numPages       ,    4);
        WRT(ppd->pCPD.mediaWidth     ,    4);
        WRT(ppd->pCPD.mediaHeight    ,    4);
        WRT(ppd->pCPD.mediaColor     ,    4);

        WRT(ppd->pCPD.orientation    ,    4);
        WRT(ppd->pCPD.sides          ,    4);

        WRT(ppd->pCPD.location.latitude          ,    4);
        WRT(ppd->pCPD.location.longitude         ,    4);
        WRT(ppd->pCPD.location.altitude          ,    4);
        WRT(ppd->pCPD.location.namedLocation     , 1024);
        WRT(ppd->pCPD.location.localNetworkDomain,    4);

        WRT(ppd->pCPD.time.year      ,    2);
        WRT(ppd->pCPD.time.month     ,    2);
        WRT(ppd->pCPD.time.day       ,    2);
        WRT(ppd->pCPD.time.hour      ,    2);
        WRT(ppd->pCPD.time.minutes   ,    2);
        WRT(ppd->pCPD.time.seconds   ,    2);

    WRT(ppd->numPages        ,    4);
    WRT(ppd->ppImageWidth    ,    4);
    WRT(ppd->ppImageHeight   ,    4);
    WRT(ppd->mediaWidth      ,    4);
    WRT(ppd->mediaHeight     ,    4);
    WRT(ppd->mediaColor      ,    4);
    WRT(ppd->pCPD.orientation,    4);

/*** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/*** What happened - Error occurred                                 */
/*** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
        break;
    }

/*** Return to Caller                                               */
/*** -------------------------------------------------------------- */
    return(status);

} /* _WritePpfHeader                                                */
#endif

/*________________________  gp.photo.dat.c   _______________________*/

/*345678901234567890123456789012345678901234567890123456789012345678*/
/*       1         2         3         4         5         6        */
/*********************************|**********************************/

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*** End of File *** End of File *** End of File *** End of File ****/
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
